<?php

namespace Ardent\Undercurrent\Http;

use Ardent\Undercurrent\Container\ClassNotFoundException;
use Ardent\Undercurrent\Container\ContainerInterface;
use Exception;
use ReflectionMethod;

class GenericRouter implements RouterInterface
{
    public function __construct(
        private readonly ContainerInterface $container,
        private readonly RouterConfig       $config,
    )
    {
    }

    /**
     * @throws RouteNotFoundException|ClassNotFoundException
     */
    public function dispatch(RequestInterface $request): ResponseInterface
    {
        foreach ($this->config->getRoutes() as $route) {
            if ($route->getRoute()->path === $request->getUri()) {
                $controller = $this->container->get($route->getController());
                $method = $route->getMethod();
                return $controller->$method(...$this->autowire($route));
            }
        }

        throw new RouteNotFoundException($request);
    }

    private function autowire(RouteConfig $route): array
    {
        $reflectionMethod = new ReflectionMethod($route->getController(), $route->getMethod());

        $params = [];
        foreach ($reflectionMethod->getParameters() as $parameter) {
            $type = $parameter->getType();
            if (!$type) {
                throw new Exception('Cannot autowire parameter without type');
            }
            $params[] = $this->container->get($type->getName());
        }

        return $params;
    }
}